@@ -1,7 +1,6 @@
 use std::mem;
 
-use Result;
-use tape::{Tape, Walue};
+use {Result, Tape, Walue};
 
 /// An offset.
 #[derive(Clone, Copy, Debug, Eq, Ord, PartialEq, PartialOrd)]
@@ -33,7 +32,7 @@ impl Walue<OffsetSize> for Offset {
             1 => try!(tape.take::<u8>()) as u32,
             2 => try!(tape.take::<u16>()) as u32,
             3 => {
-                let trio: [u8; 3] = read!(tape, 3);
+                let trio: [u8; 3] = read_value!(tape);
                 unsafe { mem::transmute::<_, u32>(assemble!(trio[0], trio[1], trio[2])) }
             },
             4 => try!(tape.take::<u32>()),
@@ -44,10 +43,11 @@ impl Walue<OffsetSize> for Offset {
 
 #[cfg(test)]
 mod tests {
-    use tape::Walue;
-    use compact::Offset;
     use std::io::Cursor;
 
+    use Walue;
+    use compact::Offset;
+
     #[test]
     fn read() {
         let mut tape = Cursor::new(vec![1, 2, 3, 4, 5, 6, 7, 8, 9, 10]);
