@@ -12,6 +12,7 @@ pub struct Program<'l> {
     local: &'l [Vec<u8>],
     stack: Vec<Operand>,
     stems: usize,
+    width: Option<Operand>,
 }
 
 struct Routine<'l> {
@@ -30,10 +31,12 @@ impl<'l> Program<'l> {
             local: local,
             stack: vec![],
             stems: 0,
+            width: None,
         }
     }
 
     /// Return the next operation.
+    #[allow(unused_comparisons)]
     pub fn next(&mut self) -> Result<Option<Operation>> {
         use type2::operation::Operator::*;
 
@@ -41,33 +44,34 @@ impl<'l> Program<'l> {
             return Ok(None);
         }
 
-        macro_rules! clear(
-            () => (mem::replace(&mut self.stack, vec![]));
-        );
         macro_rules! pop(
             () => (match self.stack.pop() {
                 Some(value) => value,
-                _ => raise!("expected an argument"),
+                _ => raise!("expected an operand"),
             });
             (bool) => (match self.stack.pop() {
                 Some(value) => value != 0.0,
-                _ => raise!("expected an argument"),
+                _ => raise!("expected an operand"),
             });
             (i32) => (match self.stack.pop() {
                 Some(value) if value as i32 as Operand == value => value as i32,
-                _ => raise!("expected an argument of a different type"),
+                _ => raise!("expected an operand of a different type"),
             });
         );
         macro_rules! push(
-            ($argument:expr) => ({
-                let argument = $argument;
-                self.stack.push(argument);
+            ($operand:expr, bool) => ({
+                let operand = $operand;
+                self.stack.push(if operand { 1.0 } else { 0.0 });
+            });
+            ($operand:expr) => ({
+                let operand = $operand;
+                self.stack.push(operand);
             });
         );
         macro_rules! read(($index:expr) => ({
             let length = self.stack.len();
             if $index >= length {
-                raise!("expected more arguments");
+                raise!("expected more operands");
             }
             self.stack[length - 1 - $index]
         }));
@@ -85,13 +89,69 @@ impl<'l> Program<'l> {
         } else {
             try!(Operator::from(try!(self.routine.take::<u8>()) as u16))
         };
+
+        macro_rules! clear(
+            (@reduce [$min:expr, $left:expr] []) => ({
+                if $min > $left {
+                    $min = $left;
+                }
+            });
+            (@reduce [$min:expr, $left:expr] [equal($count:expr), $($tail:tt)*]) => ({
+                if $left >= $count {
+                    let left = $left - $count;
+                    clear!(@reduce [$min, left] [$($tail)*]);
+                }
+            });
+            (@reduce [$min:expr, $left:expr] [maybe_equal($count:expr), $($tail:tt)*]) => ({
+                clear!(@reduce [$min, $left] [$($tail)*]);
+                if $left >= $count {
+                    clear!(@reduce [$min, $left - $count] [$($tail)*]);
+                }
+            });
+            (@reduce [$min:expr, $left:expr] [modulo($count:expr), $($tail:tt)*]) => ({
+                for i in 1..($left / $count + 1) {
+                    let left = $left - i * $count;
+                    clear!(@reduce [$min, left] [$($tail)*]);
+                }
+            });
+            (@reduce [$min:expr, $left:expr] [maybe_modulo($count:expr), $($tail:tt)*]) => ({
+                for i in 0..($left / $count + 1) {
+                    let left = $left - i * $count;
+                    clear!(@reduce [$min, left] [$($tail)*]);
+                }
+            });
+            ($([$($predicate:ident($count:expr)),*]),+) => ({
+                let length = self.stack.len();
+                let mut min = !0;
+                $(clear!(@reduce [min, length] [$($predicate($count),)*]);)+
+                if min == !0 {
+                    raise!("found malformed operands");
+                }
+                let mut stack = mem::replace(&mut self.stack, vec![]);
+                let operands = stack.drain(min..).collect();
+                if min > 0 && self.width.is_none() {
+                    self.width = Some(stack[min - 1]);
+                }
+                return Ok(Some((operator, operands)));
+            });
+        );
+
         match operator {
             // Path-construction operators
-            RMoveTo | HMoveTo | VMoveTo | RLineTo | HLineTo | VLineTo |
-            RRCurveTo | HHCurveTo | HVCurveTo | VHCurveTo | VVCurveTo |
-            RCurveLine | RLineCurve | Flex | Flex1 | HFlex | HFlex1 => {
-                return Ok(Some((operator, clear!())));
-            },
+            RMoveTo => clear!([equal(2)]),
+            HMoveTo | VMoveTo => clear!([equal(1)]),
+            RLineTo => clear!([modulo(2)]),
+            HLineTo | VLineTo => clear!([equal(1), maybe_modulo(2)], [modulo(2)]),
+            RRCurveTo => clear!([modulo(6)]),
+            HHCurveTo | VVCurveTo => clear!([maybe_equal(1), modulo(4)]),
+            HVCurveTo | VHCurveTo => clear!([equal(4), maybe_modulo(8), maybe_equal(1)],
+                                            [modulo(8), maybe_equal(1)]),
+            RCurveLine => clear!([modulo(6), equal(2)]),
+            RLineCurve => clear!([modulo(2), equal(6)]),
+            Flex => clear!([equal(13)]),
+            Flex1 => clear!([equal(11)]),
+            HFlex => clear!([equal(7)]),
+            HFlex1 => clear!([equal(9)]),
 
             // Terminal operator
             EndChar => {
@@ -101,18 +161,22 @@ impl<'l> Program<'l> {
                     }
                     mem::replace(&mut self.routine, *caller);
                 }
+                let length = self.stack.len();
+                if length > 0 && self.width.is_none() {
+                    self.width = Some(self.stack[length - 1]);
+                }
                 return Ok(None);
             },
 
             // Hint operators
             HStem | VStem | HStemHM | VStemHM => {
                 self.stems += self.stack.len() >> 1;
-                return Ok(Some((operator, clear!())));
+                clear!([equal(2), maybe_modulo(2)]);
             },
             HintMask | CntrMask => {
                 self.stems += self.stack.len() >> 1;
                 let _: Vec<u8> = read_walue!(&mut *self.routine, (self.stems + 7) >> 3);
-                return Ok(Some((operator, clear!())));
+                clear!([equal(0)]);
             },
 
             // Arithmetic operators
@@ -130,9 +194,7 @@ impl<'l> Program<'l> {
             Random => unimplemented!(),
             Mul => push!(pop!() * pop!()),
             Sqrt => push!(pop!().sqrt()),
-            Drop => {
-                pop!();
-            },
+            Drop => mem::drop(pop!()),
             Exch => {
                 let (right, left) = (pop!(), pop!());
                 push!(right);
@@ -146,15 +208,15 @@ impl<'l> Program<'l> {
                 let (shift, span) = (pop!(i32), pop!(i32));
                 let length = self.stack.len();
                 if span < 0 {
-                    raise!("found an invalid argument");
+                    raise!("found an invalid operand");
                 } else if span as usize > length {
-                    raise!("expected more arguments");
+                    raise!("expected more operands");
                 } else if span > 0 {
                     let position = length - span as usize;
                     if shift > 0 {
                         for _ in 0..shift {
-                            let argument = pop!();
-                            self.stack.insert(position, argument);
+                            let operand = pop!();
+                            self.stack.insert(position, operand);
                         }
                     } else if shift < 0 {
                         for _ in 0..(-shift) {
@@ -172,16 +234,16 @@ impl<'l> Program<'l> {
             // Conditional operators
             And => {
                 let (right, left) = (pop!(bool), pop!(bool));
-                push!(if left && right { 1.0 } else { 0.0 })
+                push!(left && right, bool);
             },
             Or => {
                 let (right, left) = (pop!(bool), pop!(bool));
-                push!(if left || right { 1.0 } else { 0.0 })
+                push!(left || right, bool);
             },
-            Not => push!(if pop!(bool) { 0.0 } else { 1.0 }),
+            Not => push!(!pop!(bool), bool),
             Eq => {
                 let (right, left) = (pop!(), pop!());
-                push!(if left == right { 1.0 } else { 0.0 })
+                push!(left == right, bool);
             },
             IfElse => {
                 let (right, left, no, yes) = (pop!(), pop!(), pop!(), pop!());
@@ -217,6 +279,12 @@ impl<'l> Program<'l> {
         };
         self.next()
     }
+
+    /// Return the width difference with respect to the nominal width.
+    #[inline]
+    pub fn width(&self) -> Option<Operand> {
+        self.width
+    }
 }
 
 impl<'l> Routine<'l> {
